/*
* Copyright 2009, KyTea Development Team
* 
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
* 
*     http://www.apache.org/licenses/LICENSE-2.0
* 
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <kytea/kytea-config.h>
#include "config.h"

using namespace kytea;
using namespace std;

// set the type of the input corpus
void KyteaConfig::setIOFormat(const char* str, CorpForm & cf) {
    if(!strcmp(str, "full"))      { cf = CorpusIO::FORMAT_FULL; }
    else if(!strcmp(str, "part")) { cf = CorpusIO::FORMAT_PART; }
    else if(!strcmp(str, "conf")) { cf = CorpusIO::FORMAT_PROB; }
    else if(!strcmp(str, "prob")) { cf = CorpusIO::FORMAT_PROB; }
    else if(!strcmp(str, "raw"))  { cf = CorpusIO::FORMAT_RAW;  }
    else {
        ostringstream buff;
        buff << "Unsupported corpus IO format '" << str << "'";
        throw runtime_error(buff.str());
    }
}


void KyteaConfig::parseTrainCommandLine(int argc, char ** argv) {
    for(int i = 1; i < argc; i++)
        i += parseTrainArg(argv[i], (i == argc-1?NULL:argv[i+1]));
}


void KyteaConfig::parseRunCommandLine(int argc, char ** argv) {
    for(int i = 1; i < argc; i++)
        i += parseRunArg(argv[i], (i == argc-1?NULL:argv[i+1]));
    // load the model file if it has not been specified at the command line
    if(model_.length() == 0) {
        if(getenv("KYTEA_MODEL"))
            model_ = getenv("KYTEA_MODEL");
        else {
            model_ = PKGDATADIR;
            model_ += "/model.bin";
        }
    }
}


void KyteaConfig::printUsage() {
    if(onTraining_) {
        // print the training usage
        cerr << 
"train-kytea:" << endl << 
"  A program to train models for KyTea" << endl <<
"" << endl <<
"Input/Output Options: " << endl <<
"  -encode  The text encoding to be used for input/output (utf8/euc/sjis; default: utf8)" << endl <<
"  -full    A file of fully annotated training data (can be specified multiple times)" << endl <<
"  -part    A file of partially annotated training data (can be specified multiple times)" << endl <<
"  -conf    A file of training data annotated with confidences (can be specified multiple times)" << endl <<
"  -dict    A dictionary file (in the form of one 'word/pron' entry per line)" << endl <<
"  -subword A dictionary file of subword units. Adding this will enable unknown word PE." << endl <<
"  -model   The file to write the trained model to" << endl <<
"  -modtext Print a text model (instead of the default binary)" << endl <<
"Model Training Options (basic)" << endl <<
"  -nows    Don't train a word segmentation model" << endl <<
"  -nope    Don't train a pronunciation estimation model" << endl <<
"Model Training Options (for advanced users): " << endl <<
"  -charw   The window of characters to use on either side of a boundary for WS (default 3)" << endl <<
"  -charn   The maximum length of character n-grams to use for WS (default 3)" << endl <<
"  -typew   The window of character types to use on either side of a boundary for WS (default 3)" << endl <<
"  -typen   The maximum length of character type n-grams to use for WS (default 3)" << endl <<
"  -dictn   All dictionary words greater than this length will be bucketed together (default 4)" << endl <<
"  -unkn    The order of the language model to use for unknown words (default 3)" << endl <<
"  -eps     The epsilon stopping criterion for classifier training" << endl <<
"  -bias    Whether to use a bias value in classifier training (default true)" << endl <<
"  -solver  The solver (1 = SVM, 6 = logistic regression, etc.; default 1)" << endl << endl;
    }
    else {
        // print the testing usage
        cerr << 
"kytea:" << endl << 
"  KyTea, a word segmentation/pronunciation estimation tool" << endl <<
"" << endl <<
"Options: " << endl <<
"  -model   The model file to use when analyzing text" << endl <<
"  -in      The formatting of the input  (raw/full/part/conf, default raw)" << endl <<
"  -out     The formatting of the output (full/part/conf, default full)" << endl <<
"  -nows    Don't do word segmentation (raw input cannot be accepted)" << endl <<
"  -nope    Don't do pronunciation estimation (full input cannot be accepted)" << endl <<
"  -nounk   Don't estimate the pronunciation of unkown words" << endl <<
"  -unkcount The maximum number of unknown pronunciations to print (default 5, 0 implies no limit)" << endl << 
"  -unktag  A tag to append to indicate words not in the dictionary" << endl << 
"  -unkbeam The width of the beam to use in beam search for unknown words (default 50, 0 for full search)" << endl << endl;
    }
    exit(1);
}

void KyteaConfig::printVersion() {
    cerr << "KyTea version "<<VERSION<<endl;
    exit(0);
}

void KyteaConfig::ch(const char * n, const char* v) {
    if(!v) {
        cerr << "Stray '" << n << "' argument" << endl << endl;
        printUsage();
    }
}       
// parse a single argument
//  the value argument can be null
//  return 1 if the value was used 0 if not
unsigned KyteaConfig::parseTrainArg(const char * n, const char * v) {
    unsigned r=1;

    if(!strcmp(n, "--help") || !strcmp(n,"-help")) { printUsage(); }
    else if(!strcmp(n, "--version") || !strcmp(n,"-version")) { printVersion(); }

    // general input/output option
    else if(!strcmp(n, "-encode"))        { ch(n,v); setEncoding(v); }

    // input options for training
    else if(!strcmp(n, "-full"))     { ch(n,v); addCorpus(v, CorpusIO::FORMAT_FULL); }
    else if(!strcmp(n, "-part"))     { ch(n,v); addCorpus(v, CorpusIO::FORMAT_PART); }
    else if(!strcmp(n, "-conf"))     { ch(n,v); addCorpus(v, CorpusIO::FORMAT_PROB); }
    else if(!strcmp(n, "-dict"))     { ch(n,v); addDictionary(v); }
    else if(!strcmp(n, "-subword"))  { ch(n,v); addSubwordDict(v); }

    // output option for training
    else if(!strcmp(n, "-model"))    { ch(n,v); setModelFile(v); }
    else if(!strcmp(n, "-modtext"))  { setModelFormat('T'); r=0; }

    // liblinear options
    else if(!strcmp(n, "-eps"))      { ch(n,v); setEpsilon(util_->parseFloat(v)); }
    else if(!strcmp(n, "-solver"))   { ch(n,v); setSolverType(util_->parseInt(v)); }

    // feature options
    else if(!strcmp(n, "-charw"))    { ch(n,v); setCharWindow(util_->parseInt(v)); }
    else if(!strcmp(n, "-charn"))    { ch(n,v); setCharN(util_->parseInt(v)); }
    else if(!strcmp(n, "-typew"))    { ch(n,v); setTypeWindow(util_->parseInt(v)); }
    else if(!strcmp(n, "-typen"))    { ch(n,v); setTypeN(util_->parseInt(v)); }
    else if(!strcmp(n, "-dictn"))    { ch(n,v); setDictionaryN(util_->parseInt(v)); }
    else if(!strcmp(n, "-unkn"))    { ch(n,v); setUnkN(util_->parseInt(v)); }
    
    // whether or not to perform word segmentation, pronunciation estimation
    else if(!strcmp(n, "-nows"))     { setDoWS(false); r=0; }
    else if(!strcmp(n, "-nope"))     { setDoPE(false); setDoUnk(false); r=0; }
    else if(!strcmp(n, "-nobias"))   { setBias(false); r=0; }

    // --- DEPRECATED ---
    // do not use these undocumented options, as they may disappear in the future
    else if(!strcmp(n, "-prob"))     { ch(n,v); addCorpus(v, CorpusIO::FORMAT_PROB); }
    else if(!strcmp(n, "-dicn"))    { ch(n,v); setDictionaryN(util_->parseInt(v)); }
    
    else if(n[0] == '-') {
        cerr << "Invalid argument '" << n << "'" << endl << endl;
        printUsage();
    }

    else { r=0; args_.push_back(n); }
    return r;
}

unsigned KyteaConfig::parseRunArg(const char * n, const char * v) {
    unsigned r=1;

    if(!strcmp(n, "--help") || !strcmp(n,"-help")) { printUsage(); }
    else if(!strcmp(n, "--version") || !strcmp(n,"-version")) { printVersion(); }

    // general input/output option
    else if(!strcmp(n, "-in"))       { ch(n,v); setIOFormat(v, inputForm_);  }
    else if(!strcmp(n, "-out"))      { ch(n,v); setIOFormat(v, outputForm_); }

    // output option for training
    else if(!strcmp(n, "-model"))    { ch(n,v); setModelFile(v); }

    // whether or not to perform word segmentation, pronunciation estimation
    else if(!strcmp(n, "-nows"))     { setDoWS(false); r=0; }
    else if(!strcmp(n, "-nope"))     { setDoPE(false); r=0; }
    else if(!strcmp(n, "-nounk"))    { setDoUnk(false); r=0; }

    // the limit on the number of unknown words to output
    else if(!strcmp(n, "-unkcount")) { ch(n,v); setUnkCount(util_->parseInt(v)); }
    else if(!strcmp(n, "-unktag"))   { ch(n,v); setUnkTag(v); }
    else if(!strcmp(n, "-unkbeam"))  { ch(n,v); setUnkBeam(util_->parseInt(v)); }

    else if(n[0] == '-') {
        cerr << "Invalid argument '" << n << "'" << endl << endl;
        printUsage();
    }

    else { r=0; args_.push_back(n); }
    return r;
}
